#include "CTTMax7219Lib.h"

/*************************************************************************************
 *
 * $RCSfile: CTTMax7219Lib.cpp,v $
 * $Date: 2014/07/20 02:16:38 $
 * $Revision: 1.1 $
 *
 * Implementation of a basic library for the Max7219CNG display driver.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * (c)2014, Claude "Tryphon" Theroux, Montreal, Quebec, 
 * 
 ************************************************************************************/

  
//*** CTTMax7219C ************************************************************

CTTMax7219::CTTMax7219(int pCLK, int pCS, int pDIN)
{
  m_CLK = pCLK;
  m_CS = pCS;
  m_DIN = pDIN;
}

//*** initialize ************************************************************

void CTTMax7219::initialize()
{
  pinMode(m_CLK, OUTPUT);
  pinMode(m_CS, OUTPUT);
  pinMode(m_DIN, OUTPUT);
  delay(100);
  write(ADDRESS_DECODE_MODE, BCDMode);      // Decode BCD.
  write(ADDRESS_BRIGHTNESS, m_Brightness);  // Brightness.
  write(0x0b, 0x07);       //
  write(0x0c, 0x01);       //
  write(ADDRESS_TEST_MODE, 0x00);           // Disable test mode.
  test(true); 
  delay(2000);
  test(false);
}

//*** setBrightness ************************************************************

void CTTMax7219::setBrightness(byte pBrightness)
{
    m_Brightness = pBrightness & 0x0f;
    write(ADDRESS_BRIGHTNESS, m_Brightness);
}

//*** test ************************************************************

void CTTMax7219::test(bool pEnable = true)
{
    if( pEnable ) {
      write(ADDRESS_TEST_MODE, 0x01);
    } else {
      write(ADDRESS_TEST_MODE, 0x00);
    }
}

//*** shutdown ************************************************************

void CTTMax7219::shutdown(bool pEnable)
{
    if( pEnable ) {
      write(ADDRESS_SHUTDOWN, 0x00);
    } else {
      write(ADDRESS_SHUTDOWN, 0x01);
    }
}

//*** setBCDMode ************************************************************

void CTTMax7219::setBCDMode(BCDModes pBCDMode) {
  BCDMode = pBCDMode;
  write(ADDRESS_DECODE_MODE, BCDMode);
}

//*** setScanLimit ************************************************************

void CTTMax7219::setScanLimit(byte pScanLimit) {
  scanLimit = (pScanLimit - 1) & 0x7;
  write(ADDRESS_SCAN_LIMIT, scanLimit);
}


//*** writeByte ************************************************************

void CTTMax7219::writeByte(unsigned char pByte) 
{   
  unsigned char i;
  digitalWrite(m_CS, LOW);		
  for(unsigned int i = 8 ; i >= 1; i--)
  {		  
    digitalWrite(m_CLK, LOW);
    digitalWrite(m_DIN, pByte & 0x80);
    pByte = pByte << 1;
    digitalWrite(m_CLK, HIGH);
  }                                 
}

//*** write ************************************************************

void CTTMax7219::write(unsigned char pAddress, unsigned char pByte)
{
        digitalWrite(m_CS,LOW);
        writeByte(pAddress);
        writeByte(pByte);
        digitalWrite(m_CS,HIGH);
}
